package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.util.ObjectUtil;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.ride.*;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.TaskQueue;
import com.singularity.pojo.vo.ride.CompareRideDataVO;
import com.singularity.pojo.vo.ride.QueryRideDataVO;
import com.singularity.pojo.vo.ride.QueryRideFileListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.RideDataService;
import com.singularity.service.TaskQueueService;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Slf4j
@Api(tags = "平顺性相关接口")
@RestController
@RequestMapping("api/ride")
public class RideController {

    @Resource
    private RideDataService rideDataService;

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private TaskQueueService taskQueueService;

    @Value("${file.path}")
    private String filePath;


    @ApiOperation("上传平顺性试验数据文件")
    @LogOperation(message = "上传平顺性试验数据文件")
    @PostMapping("/uploadFile")
    public Result<?> uploadFile(@Validated UploadRideFileDTO dto) {
        MultipartFile multipartFile = dto.getMultipartFile();
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getCarId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        // 批量定时上传
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
                dto.setSize(multipartFile.getSize());
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(
                    () -> rideDataService.uploadFileList(dto),dto.getStartTime()
            );
        }else{
            // 批量上传
            rideDataService.uploadFileList(dto);
        }

        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("平顺性文件列表查询")
    @PostMapping("/queryRideFileList")
    public Result<ResponseDTOWithPage<QueryRideFileListVO>> queryRideFileList(@RequestBody @Validated QueryRideFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                attachmentService.queryRideFileList(dto));
    }

    @SaCheckRole("admin")
    @ApiOperation("批量删除平顺性试验数据文件")
    @LogOperation(message = "批量删除平顺性试验数据文件")
    @PostMapping("/deleteFile")
    public Result<?> deleteFile(@Validated @RequestBody DeleteFileDTO dto) {
        rideDataService.deleteFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询平顺性试验结果")
    @LogOperation(message = "查询平顺性试验结果")
    @PostMapping("/queryRideData")
    public Result<List<QueryRideDataVO>> queryRideData(@RequestBody @Validated QueryRideDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, rideDataService.queryRideData(dto));
    }

    @ApiOperation("修改平顺性试验结果")
    @LogOperation(message = "修改平顺性试验结果")
    @PostMapping("/updateRideData")
    public Result<?> updateRideData(@RequestBody @Validated UpdateRideDataDTO dto) {
        rideDataService.updateRideData(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("对比平顺性试验结果")
    @LogOperation(message = "对比平顺性试验结果")
    @PostMapping("/compareRideData")
    public Result<CompareRideDataVO> compareRideData(@RequestBody @Validated CompareRideDataDTO dto) {

        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, rideDataService.compareRideData(dto));
    }

    @ApiOperation("行驶性能平顺性文件列表分页查询")
    @PostMapping(value = "/queryRideFileReportList")
    public Result<List<QueryAttachmentListVO>> queryRideFileReportList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(dto.getBusinessId());
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @ApiOperation("上传行驶性能试验报告平顺性文件")
    @LogOperation(message = "上传行驶性能试验报告平顺性文件")
    @PostMapping(value = "/uploadRideFileReport")
    public Result<List<QueryAttachmentListVO>> uploadRideFileReport(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        dto.setBusinessType(FileBusinessConstant.RIDE_COMFORT_REPORT);
        attachmentService.uploadAttachment(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能试验报告平顺性文件")
    @LogOperation(message = "删除行驶性能试验报告平顺性文件")
    @PostMapping(value = "/deleteRideFileReport")
    public Result<List<QueryAttachmentListVO>> deleteRideFileReport(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("上传平顺性原始数据文件")
    @LogOperation(message = "上传平顺性原始数据文件")
    @PostMapping("/uploadRideReSourceFile")
    public Result<List<Attachment>> uploadRideReSourceFile(@Validated UploadRideReSourceFileDTO dto) {
        MultipartFile multipartFile = dto.getMultipartFile();
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getOwnerId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        // 批量定时上传
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
                dto.setSize(multipartFile.getSize());
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(
                    () -> rideDataService.uploadRideReSourceFile(dto),dto.getStartTime()
            );
        }else{
            // 批量上传
            rideDataService.uploadRideReSourceFile(dto);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

}
