package com.singularity.handler;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import cn.dev33.satoken.exception.NotRoleException;
import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.util.SaResult;
import com.singularity.exception.CommonErrorEnum;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.util.result.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

/**
 * 全局异常处理器
 */
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    /**
     * 捕获@validate参数校验异常
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public SaResult MethodArgumentNotValidExceptionHandler(HttpServletRequest request, MethodArgumentNotValidException ex) {
        final BindingResult bindingResult = ex.getBindingResult();
        StringBuilder errMsg = new StringBuilder(bindingResult.getFieldErrors().size() * 16);
        for (int i = 0; i < bindingResult.getFieldErrors().size(); i++) {
            if (i > 0) {
                errMsg.append(",");
            }
            FieldError error = bindingResult.getFieldErrors().get(i);
            errMsg.append(error.getDefaultMessage());
        }
        log.error("{} 传参错误：{}", request.getRequestURI(), errMsg);
        return SaResult.error(errMsg.toString());
    }

    /**
     * 参数类型错误异常
     */
    @ExceptionHandler(HttpMessageConversionException.class)
    public SaResult parameterTypeException(HttpServletRequest request, HttpMessageConversionException ex) {
        log.error("{} 捕获异常：{}", request.getRequestURI(), ex.getCause().getLocalizedMessage());
        return SaResult.error(ex.getCause().getLocalizedMessage());
    }


    /**
     * 未知异常或线程池异常
     */
    @ExceptionHandler(value = Exception.class)
    public SaResult handlerException(Exception e) {
        e.printStackTrace();
        log.error("system exception! userId: {} The reason is: {}", StpUtil.getLoginId().toString(), e.getMessage(), e);
        return SaResult.error(CommonErrorEnum.SYSTEM_ERROR.getMsg());
    }

    /**
     * 未登录异常
     */
    @ExceptionHandler(value = NotLoginException.class)
    public SaResult handlerNotLoginException(NotLoginException e) {
        log.error("not login exception! The reason is: {}", e.getMessage(), e);
        return new SaResult(e.getCode(), e.getMessage().split("：")[0], null);
    }

    /**
     * 空指针异常
     */
    @ExceptionHandler(value = NullPointerException.class)
    public SaResult handlerNullPointerException(NullPointerException e) {
        log.error("null point exception！The reason is: {}", e.getMessage(),e);
        return SaResult.error(CommonErrorEnum.SYSTEM_ERROR.getMsg());
    }

    /**
     * 业务异常
     */
    @ExceptionHandler(value = SaTokenException.class)
    public SaResult handlerSaTokenException(SaTokenException e) {
        e.printStackTrace();
        return new SaResult(e.getCode(),e.getMessage(),null);
    }

    /**
     * 没有角色权限异常
     */
    @ExceptionHandler(value = NotRoleException.class)
    public SaResult handlerSaTokenException(NotRoleException e) {
        e.printStackTrace();
        return new SaResult(500,"对不起,您没有此操作权限",null);
    }

    /**
     * 权限异常
     */
    @ExceptionHandler(value = NotPermissionException.class)
    public SaResult handlerNotPermissionException(NotPermissionException e){
        log.error("noy permission exception！The reason is: {}", e.getMessage(),e);
        return new SaResult(CommonErrorEnum.PERMISSION_ERROR.getCode(),e.getMessage(),null);
    }

    /**
     * 自定义运行时异常
     * 此异常需由自己抛出
     */
    @ExceptionHandler(ToolsBusinessWarn.class)
    public Result<?> ExceptionHandler(HttpServletRequest request, Exception ex) {
        ToolsBusinessWarn businessWarn = (ToolsBusinessWarn) ex;
        log.error("{} 捕获异常：{}", request.getRequestURI(), ex.getMessage());
        return new Result<>(Result.OPERATION_RESULT_TYPE.success, businessWarn.getCode(), ex.getMessage(),null);
    }
}
