package com.singularity.service.impl;

import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.dao.PermissionMapper;
import com.singularity.dao.SysMenuMapper;
import com.singularity.pojo.dto.permission.MenuDeleteDTO;
import com.singularity.pojo.dto.permission.MenuSetDTO;
import com.singularity.pojo.dto.permission.PermissionInfoDTO;
import com.singularity.pojo.dto.permission.queryMenuTrueListDTO;
import com.singularity.pojo.dto.role.RoleInfoDTO;
import com.singularity.pojo.entity.Permission;
import com.singularity.pojo.entity.SysMenu;
import com.singularity.pojo.vo.permission.QueryMenuDropListVO;
import com.singularity.pojo.vo.permission.QueryMenuTrueListVO;
import com.singularity.service.MenuRoleService;
import com.singularity.service.PermissionService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class PermissionServiceImpl implements PermissionService {

    private final PermissionMapper permissionMapper;

    private final SysMenuMapper sysMenuMapper;

    private final MenuRoleService menuRoleService;

    @Autowired
    public PermissionServiceImpl(PermissionMapper permissionMapper, SysMenuMapper sysMenuMapper,MenuRoleService menuRoleService) {
        this.permissionMapper = permissionMapper;
        this.sysMenuMapper = sysMenuMapper;
        this.menuRoleService = menuRoleService;
    }

    @Override
    public List<PermissionInfoDTO> getPermissionInfo(List<RoleInfoDTO> roles) {
        List<Long> roleIds = roles.stream().map(RoleInfoDTO::getId).collect(Collectors.toList());
        //查询所有权限
        List<PermissionInfoDTO> permissionAllInfo = permissionMapper.selectAll(1);
        if (roleIds.isEmpty()) {
            throw new SaTokenException("该用户角色为空，请联系管理员添加！");
        }

        //查询角色拥有权限
        List<PermissionInfoDTO> permissionInfoDTOList = permissionMapper.selectByRoleIds(roleIds);

        return this.buildPermissionsInfoTree(permissionInfoDTOList, permissionAllInfo);
    }

    @Override
    public List<PermissionInfoDTO> getAllPermissionList(Integer status) {
        //查询所有权限
        List<PermissionInfoDTO> permissionAllInfo = permissionMapper.selectAll(status);

        //获取顶层权限
        List<PermissionInfoDTO> permissionInfoDTOList = permissionAllInfo.stream()
                .filter(permissionInfoDTO -> permissionInfoDTO.getPId() == 0).collect(Collectors.toList());
        return this.buildPermissionsInfoTree(permissionInfoDTOList, permissionAllInfo);
    }

    @Override
    public List<PermissionInfoDTO> getPermissionList(Long roleId) {
        //查询所有权限
        List<PermissionInfoDTO> permissionAllInfo = permissionMapper.selectAll(1);

        //查询角色拥有权限
        List<PermissionInfoDTO> permissionInfoDTOList = permissionMapper.selectByRoleId(roleId);
        return this.buildPermissionsInfoTree(permissionInfoDTOList, permissionAllInfo);
    }

    @Override
    public Boolean insert(Permission permission) {
        return permissionMapper.insert(permission) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean update(Permission permission) {
        return permissionMapper.updateById(permission) > 0;
    }

    @Override
    public List<QueryMenuTrueListVO> queryMenuTrueList(queryMenuTrueListDTO dto) {
        List<QueryMenuTrueListVO> queryMenuTrueList = sysMenuMapper.queryMenuTrueList(dto);
        return this.convertMenuTree(queryMenuTrueList);
    }

    /**
     * 两层循环封装菜单树
     *
     * @param treeNodes 需要封装的菜单集合
     */
    private List<QueryMenuTrueListVO> convertMenuTree(List<QueryMenuTrueListVO> treeNodes) {
        List<QueryMenuTrueListVO> trees = new ArrayList<>();
        for (QueryMenuTrueListVO treeNode : treeNodes) {
            if (Objects.equals(treeNode.getParentId(), (long)0)) {
                trees.add(treeNode);
            }
            for (QueryMenuTrueListVO it : treeNodes) {
                if (Objects.equals(treeNode.getId(), it.getParentId())) {
                    if (treeNode.getChildren() == null || treeNode.getChildren().size() == 0) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    List<QueryMenuTrueListVO> treeNodeChildren = treeNode.getChildren();
                    treeNodeChildren.add(it);
                    treeNode.setChildren(treeNodeChildren);
                }
            }
        }
        return trees;
    }

    private void setTreeChildrenEmptyDfs(QueryMenuTrueListVO root) {
        if (root.getChildren().size() == 0){
            root.setChildren(null);
            return;
        }
        root.getChildren().forEach(i->setTreeChildrenEmptyDfs(i));
    }
    @Override
    public void addMenu(MenuSetDTO menuSetDTO) {
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(menuSetDTO,sysMenu);
        sysMenu.setLevel(1);
        sysMenu.setStat(1);
        sysMenuMapper.insert(sysMenu);
    }

    @Override
    public void setMenu(MenuSetDTO menuSetDTO) {
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(menuSetDTO,sysMenu);
        if(ObjectUtil.isEmpty(menuSetDTO.getParentId()))
            sysMenu.setParentId((long)0);
        sysMenuMapper.updateById(sysMenu);
    }

    @Override
    public void deleteMenu(MenuDeleteDTO menuDeleteDTO) {
        sysMenuMapper.deleteBatchIds(menuDeleteDTO.getIdList());
    }

    @Override
    public List<QueryMenuDropListVO> queryMenuDropList() {
        LambdaQueryWrapper<SysMenu> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysMenu::getStat,1)
        .eq(SysMenu::getLevel,1);
        return BeanUtil.copyToList(sysMenuMapper.selectList(queryWrapper),QueryMenuDropListVO.class);
    }

    private void getChildrenDfs(QueryMenuTrueListVO root, List<QueryMenuTrueListVO> allList) {
        root.setChildren(
                allList.stream().filter(item ->
                        item.getParentId().equals(root.getId())
                ).collect(Collectors.toList())
        );
        root.getChildren().forEach(i->getChildrenDfs(i,allList));
    }

    /**
     * @param permissionInfoDTOList 角色权限
     * @param permissionAllInfo     所有权限
     * @return
     */
    private List<PermissionInfoDTO> buildPermissionsInfoTree(List<PermissionInfoDTO> permissionInfoDTOList, List<PermissionInfoDTO> permissionAllInfo) {
        // 遍历角色拥有权限
        permissionInfoDTOList.forEach(permissionInfoDTO -> {
            List<PermissionInfoDTO> chilePermissionInfoDTOList = permissionInfoDTO.getPermissions();
            // 遍历所有权限
            permissionAllInfo.forEach(childPermissionInfoDTO -> {
                // 判断权限之前的关系
                if (childPermissionInfoDTO.getPId().equals(permissionInfoDTO.getId())) {
                    chilePermissionInfoDTOList.add(childPermissionInfoDTO);
                }
            });
            // 递归构建权限树
            buildPermissionsInfoTree(chilePermissionInfoDTOList, permissionAllInfo);
        });
        return permissionInfoDTOList;
    }

}
