package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.ride.*;
import com.singularity.dao.RideDataMapper;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.enumeration.ExperimentEnum;
import com.singularity.pojo.vo.ride.CompareRideDataVO;
import com.singularity.pojo.vo.ride.QueryRideDataVO;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import com.singularity.listener.RideReadPulseDataListener;
import com.singularity.listener.RideReadRandomDataListener;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@Slf4j
@Service
public class RideDataServiceImpl extends ServiceImpl<RideDataMapper, RideData> implements RideDataService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private ExperimentService experimentService;

    @Resource(name = "threadPool")
    private ThreadPoolTaskExecutor threadPool;

    @Resource
    private TaskQueueService taskQueueService;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private CarBasicParamService carBasicParamService;

    /**
     * 批量上传平顺性试验数据文件
     */
    @Override
    @Transactional
    @SneakyThrows
    public void uploadFileList(UploadRideFileDTO dto) {
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try {
            if(multipartFile.getOriginalFilename().contains(".zip")){ //批量上传
                InputStream inputStream = multipartFile.getInputStream();
                ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
                ZipEntry zipEntry = zipInputStream.getNextEntry();
                while (zipEntry != null) {
                    if (!zipEntry.isDirectory()) {
                        String[] currentFilePath = zipEntry.getName().split("/");
                        String fileName = currentFilePath[currentFilePath.length - 1];
                        dto.setOriginalFilename(fileName);
                        byte[] bytes = IOUtils.toByteArray(zipInputStream);
                        InputStream zipEntryStream = new ByteArrayInputStream(bytes);
                        // 保存试验场景信息
                        // 获取文件名称
                        if(currentFilePath.length != 3)
                            throw new ToolsBusinessWarn("压缩包格式错误，请使用导入模板导入数据");
                        String[] experiment = currentFilePath[currentFilePath.length - 2].split("_");
                        dto.setCarLoad(ExperimentEnum.DrivingModeEnum.getCodeByName(experiment[1]));
                        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
                        dto.setMultipartFile(new MockMultipartFile(fileName, fileName, fileType, zipEntryStream));
                        this.readRideTestData(dto);
                    }
                    zipInputStream.closeEntry();
                    zipEntry = zipInputStream.getNextEntry();
                }
                zipInputStream.close();
                inputStream.close();
            }else{ //单个上传
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                dto.setSize(multipartFile.getSize());
                this.readRideTestData(dto);
            }
        }catch(Exception e){
            e.printStackTrace();
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
    }

    /**
     * 读取平顺性试验结果与试验曲线数据
     * @param dto
     * @throws Exception
     */
    private void readRideTestData(UploadRideFileDTO dto) throws Exception{
        MultipartFile multipartFile = dto.getMultipartFile();
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getOriginalFilename(),
                dto.getCarId(),
                37,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 将文件上传至minio
        Attachment attachment = attachmentService.uploadAttachment(
                multipartFile.getInputStream(),
                dto.getOriginalFilename(),
                multipartFile.getSize(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                37,
                dto.getOperId()
        );

        // 保存试验场景信息
        ExperimentInfo experimentInfo = ExperimentInfo.builder()
                .attachId(attachment.getId())
                .carLoad(dto.getCarLoad())
                .build();
        experimentService.save(experimentInfo);

        // 读取excel文件
        List<RideData> rideDataList;
        // 脉冲输入-Zmax
        RideReadPulseDataListener rideReadPulseDataListener = new RideReadPulseDataListener(
                attachment.getId().intValue(),
                0
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadPulseDataListener)
                .headRowNumber(1).sheet("脉冲输入-Zmax").doRead();
        rideDataList = rideReadPulseDataListener.getRideDataList();
        this.saveBatch(rideDataList);

        // 脉冲输入-VDV
        RideReadPulseDataListener rideReadPulseDataListener2 = new RideReadPulseDataListener(
                attachment.getId().intValue(),
                1
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadPulseDataListener2)
                .headRowNumber(1).sheet("脉冲输入-VDV").doRead();
        rideDataList = rideReadPulseDataListener2.getRideDataList();
        this.saveBatch(rideDataList);

        // 脉冲输入-峰值系数
        RideReadPulseDataListener rideReadPulseDataListener3 = new RideReadPulseDataListener(
                attachment.getId().intValue(),
                2
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadPulseDataListener3)
                .headRowNumber(1).sheet("脉冲输入-峰值系数").doRead();
        rideDataList = rideReadPulseDataListener3.getRideDataList();
        this.saveBatch(rideDataList);

        // 随机输入-M类
        RideReadRandomDataListener rideReadRandomDataListener = new RideReadRandomDataListener(
                attachment.getId().intValue(),
                3
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadRandomDataListener)
                .headRowNumber(1).sheet("随机输入-M类").doRead();
        rideDataList = rideReadRandomDataListener.getRideDataList();
        this.saveBatch(rideDataList);

        // 随机输入-N类
        RideReadRandomDataListener rideReadRandomDataListener2 = new RideReadRandomDataListener(
                attachment.getId().intValue(),
                4
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadRandomDataListener2)
                .headRowNumber(1).sheet("随机输入-N类").doRead();
        rideDataList = rideReadRandomDataListener2.getRideDataList();
        this.saveBatch(rideDataList);

        // 随机输入-卧姿
        RideReadRandomDataListener rideReadRandomDataListener3 = new RideReadRandomDataListener(
                attachment.getId().intValue(),
                5
        );
        EasyExcel.read(multipartFile.getInputStream(), rideReadRandomDataListener3)
                .headRowNumber(1).sheet("随机输入-卧姿").doRead();
        rideDataList = rideReadRandomDataListener3.getRideDataList();
        this.saveBatch(rideDataList);
    }

    /**
     * 批量上传平顺性原始数据文件
     */
    @Override
    public void uploadRideReSourceFile(UploadRideReSourceFileDTO dto) {
        // 若localFilePath不为空表示异步上传
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try{
            if(multipartFile.getOriginalFilename().contains(".zip")){
                attachmentService.uploadResourceFileList(
                    multipartFile,
                    AttachmentConstant.OWNER_TYPE_CAR,
                    dto.getOwnerId(),
                    FileBusinessConstant.RIDE_RESOURCE_FILE,
                    dto.getOperId()
                );
            }else{
                Attachment attachment = attachmentService.uploadAttachment(
                    multipartFile,
                    AttachmentConstant.OWNER_TYPE_CAR,
                    dto.getOwnerId(),
                    FileBusinessConstant.RIDE_RESOURCE_FILE,
                    dto.getOperId()
                );

                // 保存试验场景信息
                ExperimentInfo experimentInfo = ExperimentInfo.builder()
                        .attachId(attachment.getId())
                        .build();
                experimentService.save(experimentInfo);
            }
            taskQueue.setStat(2);
        }catch(Exception e){
            e.printStackTrace();
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setRideUploadStat(2);
        carBasicParam.setId(dto.getOwnerId());
        carBasicParamService.updateById(carBasicParam);
    }

    @Override
    public List<QueryRideDataVO> queryRideData(QueryRideDataDTO dto) {
        LambdaQueryWrapper<RideData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(RideData::getAttachId,dto.getAttachId())
                .eq(RideData::getTestType,dto.getTestType())
                .eq(RideData::getCarSpeed,dto.getCarSpeed());
        return BeanUtil.copyToList(this.list(queryWrapper),QueryRideDataVO.class);
    }

    /**
     * 修改平顺性试验结果
     */
    @Override
    public void updateRideData(UpdateRideDataDTO dto) {
        RideData rideData = new RideData();
        BeanUtil.copyProperties(dto,rideData);
        this.updateById(rideData);
    }

    /**
     * 批量删除平顺性试验数据文件
     */
    @Override
    public void deleteFile(DeleteFileDTO dto) {
        // 删除minio上的数据文件
        attachmentService.removeByIds(dto.getAttachIdList());

        // 删除数据库数据
        LambdaQueryWrapper<RideData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(RideData::getAttachId,dto.getAttachIdList());
        this.remove(queryWrapper);
    }

    /**
     * 对比平顺性试验结果
     */
    @Override
    public CompareRideDataVO compareRideData(CompareRideDataDTO dto) {
        CompareRideDataVO compareRideDataVO = new CompareRideDataVO();
        // 查询所有数据
        LambdaQueryWrapper<RideData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(RideData::getAttachId,dto.getMap().keySet());
        List<RideData> rideDataList = this.list(queryWrapper);

        // 通过附件id分组
        Map<Integer,List<RideData>> groupMap = rideDataList.stream()
                .collect(Collectors.groupingBy(RideData::getAttachId));

        // 封装表头
        for (Integer key : groupMap.keySet()){
            // 设置表头
            List<RideData> rideDataList2 = groupMap.get(key);
            List<String> tableHeadList = rideDataList2.stream()
                    .map(RideData::getTestIndex).collect(Collectors.toList());
            compareRideDataVO.setTableHeadList(tableHeadList);

            // 设置对比数据
            Map<Integer, Map<String,Double>> compareData = new HashMap<>();
            for (RideData rideData : rideDataList2) {
                Map<String,Double> data = new HashMap<>();
                data.put(rideData.getTestIndex(),rideData.getAverageData());
                int carId = dto.getMap().get(rideData.getAttachId());
                compareData.put(carId,data);
            }
            compareRideDataVO.setData(compareData);
        }
        return compareRideDataVO;
    }
}
