package com.singularity.util;

import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.util.ZipUtil;
import com.google.gson.Gson;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.text.PDFTextStripper;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;

@Slf4j
@Component
public class FileUtil {

    /**
     * 文件删除
     */
    @SneakyThrows
    public static void deleteFile(String filePathAndName) {
        Path path = Paths.get(filePathAndName);
        Files.delete(path);
    }

    /**
     * 下载文件到本地文件
     * copyInputStreamToFile 关闭了io流
     */
    public static void downloadToLocal(InputStream inputStream,String filePathANDName) throws IOException {
        File file = new File(filePathANDName);
        FileUtils.copyInputStreamToFile(inputStream,file);
    }

    /**
     * 返回文件io流
     */
    @SneakyThrows
    public static void downloadToResponse(String fileNameAndPath, String fileName, HttpServletResponse response){

        File file = new File(fileNameAndPath);
        if (file.isFile() && !file.exists()) {
            throw new SaTokenException("文件不存在");
        }
        InputStream inputStream = new FileInputStream(file);
        try {
            // 字符集编码
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            //清空response中的输出流
            response.reset();
            //设置Content-Type头
            response.setContentType("application/octet-stream;charset=UTF-8");
            //下面这行代码必须加，否则前端获取不到Content-Disposition字段，即无法获取文件名
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            BufferedInputStream bis = new BufferedInputStream(inputStream);
            BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream());
            byte[] buffer = new byte[1024];
            int length;
            while ((length = bis.read(buffer)) != -1) {
                bos.write(buffer, 0, length);
            }
            bis.close();
            bos.close();
        } catch (IOException e) {
            throw new SaTokenException("下载失败");
        }
    }

    /**
     * 使用hutool工具类打包文件并返回io流
     */
    @SneakyThrows
    public static void packageMinioFileList(HttpServletResponse response,String[] fileNames, InputStream[] inputStreams){
        try {
            // 字符集编码
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            //清空response中的输出流
            response.reset();
            //设置Content-Type头
            response.setContentType("application/octet-stream;charset=UTF-8");
            ZipUtil.zip(response.getOutputStream(), fileNames, inputStreams);
        } catch (IOException e) {
            throw new SaTokenException("下载失败");
        }
    }

    /**
     * multipartFile转File
     */
    public static File multipartFileToFile (MultipartFile multipartFile,String outPathAndName) {
        File file = new File(outPathAndName);
        InputStream inputStream = null;
        FileOutputStream outputStream = null;
        try {
            // 获取文件输入流
            inputStream = multipartFile.getInputStream();

            if (!file.exists()) {
                file.createNewFile();
            }
            // 创建输出流
            outputStream = new FileOutputStream(file);
            byte[] bytes = new byte[1024];
            int len;
            // 写入到创建的临时文件
            while ((len = inputStream.read(bytes)) > 0) {
                outputStream.write(bytes, 0, len);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            // 关流
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
                if (outputStream != null) {
                    inputStream.close();
                }
                outputStream.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return file;
    }

    /**
     * File转MultipartFile
     */
    public static MultipartFile fileToMultipartFile(String fileNameAndPath){
        try{
            String fileType = fileNameAndPath.substring(fileNameAndPath.lastIndexOf(".") + 1);
            File file = new File(fileNameAndPath);
            if (!file.exists() || file.isDirectory()) {
                throw new SaTokenException("文件格式错误");
            }
            String originalFilename = file.getName();
            FileInputStream input = new FileInputStream(file);
            return new MockMultipartFile(originalFilename, originalFilename, fileType, input);
        }catch (IOException i){
            throw new SaTokenException("文件转换失败");
        }
    }


    /**
     * 将对象序列化到文件
     */
    public static void saveObjectByObjectOutput(Object o, File file) {
        try {
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(new FileOutputStream(file));
            objectOutputStream.writeObject(o);
            objectOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 文件反序列化到对象
     */
    public static Object getObjectByObjectInput(File file) {
        try {
            ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream(file));
            Object o = inputStream.readObject();
            inputStream.close();
            return o;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 文件反序列化到对象
     */
    public static Object getObjectByObjectInput(InputStream inputStream) {
        try {
            ObjectInputStream objectInputStream = new ObjectInputStream(inputStream);
            Object o = objectInputStream.readObject();
            inputStream.close();
            return o;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 将对象转换为json字符串到本地
     */
    public static void saveObjectByJson(File file, Object o) {
        String json = new Gson().toJson(o);
        try {
            FileOutputStream fileOutputStream = new FileOutputStream(file);
            fileOutputStream.write(json.getBytes());
            fileOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 将本地json字符串转换为对象
     */
    public static Object getObjectByJson(File file, Class c) {
        try {
            FileInputStream fileInputStream = new FileInputStream(file);
            StringBuilder stringBuilder = new StringBuilder();
            int l = 0;
            byte[] bs = new byte[1024];
            while ((l = fileInputStream.read(bs)) != -1) {
                stringBuilder.append(new String(bs, 0, l));
            }
            fileInputStream.close();
            return new Gson().fromJson(stringBuilder.toString(), c);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * object 转 二进制数组
     */
    public static byte[] objectToByte(Object object) throws IOException {
        ByteArrayOutputStream byt = new ByteArrayOutputStream();
        ObjectOutputStream obj = new ObjectOutputStream(byt);
        obj.writeObject(object);
        return byt.toByteArray();
    }

    /**
     * 二进制数组 转 object
     */
    public static Object byteToObject(byte[] bytes) throws IOException, ClassNotFoundException {
        ByteArrayInputStream byteInt = new ByteArrayInputStream(bytes);
        ObjectInputStream objInt = new ObjectInputStream(byteInt);
        return objInt.readObject();
    }

    public static byte[] svgDataUrlToByte(String svgDataUrl){
        String  base64Data = svgDataUrl.split(",")[1];
        return Base64.getDecoder().decode(base64Data);
    }

    /**
     * 使用pdfbox提取PDF文本
     */
    public static String readPDFFile(InputStream inputStream) {
        try {
            PDDocument doc = PDDocument.load(inputStream);
            PDFTextStripper textStripper = new PDFTextStripper();
            return textStripper.getText(doc);
        } catch (Exception e) {
            throw new SaTokenException("读取pdf文档数据错误！");
        }
    }
}
