package com.singularity.util;

import cn.hutool.core.collection.CollectionUtil;
import com.singularity.util.authentication.CustomXWPFDocument;
import org.apache.poi.ooxml.POIXMLDocument;
import org.apache.poi.xwpf.usermodel.*;
import org.apache.xmlbeans.XmlCursor;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.*;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class WordUtil {

    /**
     * 根据文件url地址创建docx解析对象
     * 根据模板生成新word文档
     * 判断表格是需要替换还是需要插入，判断逻辑有$为替换，表格无$为插入
     *
     * @param inputUrl  模板存放地址
     * @param outputUrl 新文档存放地址
     * @param textMap   需要替换的信息集合
     * @param tableList 需要插入的表格信息集合
     * @return 成功返回true, 失败返回false
     */
    public static boolean changWord(String inputUrl, String outputUrl,
                                    Map<String, Object> textMap, List<String[]> tableList,int start, int end) {

        //模板转换默认成功
        boolean changeFlag = true;
        try {
            //获取docx解析对象
            CustomXWPFDocument document = new CustomXWPFDocument(POIXMLDocument.openPackage(inputUrl));
            //解析替换文本段落对象
            WordUtil.changeText(document, textMap);
            //解析替换表格对象
            WordUtil.changeTable(document, textMap, tableList,start,end);

            //生成新的word
            File file = new File(outputUrl);
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            FileOutputStream stream = new FileOutputStream(file);

            document.write(stream);
            stream.close();

        } catch (IOException e) {
            e.printStackTrace();
            changeFlag = false;
        }

        return changeFlag;

    }

    /**
     * 根据IO流创建docx解析对象
     * 根据模板生成新word文档
     * 替换word模板中的图片
     * 生成表格并根据字符替换表格
     *
     * @param inputStream  模板io流
     * @param outputUrl 新文档存放地址
     * @param textMap   需要替换的信息集合
     * @param tableList 需要插入的表格信息集合
     * @return 成功返回true, 失败返回false
     */
    public static boolean changWord(InputStream inputStream, String outputUrl,
                                     Map<String, Object> textMap, Map<String,List<Map<Integer,String>>> tableList) {
        //模板转换默认成功
        boolean changeFlag = true;
        try {
            //获取docx解析对象
            CustomXWPFDocument document = new CustomXWPFDocument(inputStream);

            //解析替换文本段落对象
            WordUtil.changeText(document, textMap);

            //解析替换表格对象
            WordUtil.changeTable2(document, tableList);

            //生成新的word
            File file = new File(outputUrl);
            FileOutputStream stream = new FileOutputStream(file);
            document.write(stream);
            stream.close();
        } catch (IOException e) {
            e.printStackTrace();
            changeFlag = false;
        }

        return changeFlag;

    }

    /**
     * 根据模板生成新word文档
     * 判断表格是需要替换还是需要插入，判断逻辑有$为替换，表格无$为插入
     *
     * @param inputUrl  模板位置
     * @param outputUrl 新文档存放地址
     * @param textMap   需要替换的信息集合
     */
    public static void changWord(String inputUrl, String outputUrl,
                                    Map<String, Object> textMap, List<List<Map<Integer,String>>> list) {
        try {
            //获取docx解析对象
            CustomXWPFDocument document = new CustomXWPFDocument(POIXMLDocument.openPackage(inputUrl));
            //解析替换文本段落对象
            WordUtil.changeText(document, textMap);
            //解析替换表格对象
            WordUtil.changeTable(document, textMap, list);

            //生成新的word
            File file = new File(outputUrl);
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            FileOutputStream stream = new FileOutputStream(file);


            document.write(stream);
            stream.close();

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    /**
     * 替换段落中的文字与图片
     *
     * @param document docx解析对象
     * @param textMap  需要替换的信息集合
     */
    public static void changeText(CustomXWPFDocument document, Map<String, Object> textMap) {
        if(CollectionUtil.isEmpty(textMap))
            return;
        //获取段落集合
        List<XWPFParagraph> paragraphs = document.getParagraphs();

        for (XWPFParagraph paragraph : paragraphs) {
            //判断此段落时候需要进行替换
            String text = paragraph.getText();
            if (checkText(text)) {
                List<XWPFRun> runs = paragraph.getRuns();
                for (XWPFRun run : runs) {
                    Object ob = getValueByKey(run.toString(), textMap);
                    // 替换内容为字符串
                    if (ob instanceof String) {
                        run.setText((String) ob, 0);
                    } else if (ob instanceof Map) { // 替换内容为图片
                        run.setText("", 0);

                        Map pic = (Map) ob;
                        int width = Integer.parseInt(pic.get("width").toString());
                        int height = Integer.parseInt(pic.get("height").toString());
                        int picType = getPictureType(pic.get("type").toString());
                        byte[] byteArray = (byte[]) pic.get("content");
                        ByteArrayInputStream byteInputStream = new ByteArrayInputStream(byteArray);
                        try {
                            String ind = document.addPictureData(byteInputStream, picType);
                            document.createPicture(run, ind, document.getNextPicNameNumber(picType), width, height);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        }

    }

    /**
     * 替换表格中的图片与文字
     *
     * @param document  docx解析对象
     * @param textMap   需要替换的信息集合
     * @param tableList 需要插入的表格信息集合
     */
    public static void changeTable(CustomXWPFDocument document, Map<String, Object> textMap,
                                   List<String[]> tableList,int start, int end) {
        //获取表格对象集合
        List<XWPFTable> tables = document.getTables();
        for (int i = 0; i < tables.size(); i++) {
            //只处理行数大于等于2的表格，且不循环表头
            XWPFTable table = tables.get(i);
            if (table.getRows().size() > 1) {
                //判断表格是需要替换还是需要插入，判断逻辑有$为替换，表格无$为插入
                if (checkText(table.getText())) {
                    List<XWPFTableRow> rows = table.getRows();
                    //遍历表格,并替换模板
                    eachTable(document, rows, textMap);
                } else {
                    insertTable(table, tableList,start,end);
                }
            }
        }
    }

    /**
     * 批量创建表格，并将List中的数据写入表格
     *
     * @param document  docx解析对象
     * @param tableList 需要插入的表格信息集合
     */
    public static void changeTable2(CustomXWPFDocument document, Map<String,List<Map<Integer,String>>> tableList) {
        List<XWPFParagraph> paragraphList = new ArrayList<>();
        for (String key : tableList.keySet()) {

            List<Map<Integer, String>> list = tableList.get(key);

            key = "${" + key + "}";
            // 将指定字符替换为表格
            paragraphList = document.getParagraphs();
            if (paragraphList != null && paragraphList.size() > 0) {
                for (XWPFParagraph paragraph : paragraphList) {
                    List<XWPFRun> runs = paragraph.getRuns();
                    for (int i = 0; i < runs.size(); i++) {
                        String text = runs.get(i).getText(0);
                        if (text != null) {
                            text = text.trim();
                            if (text.contains(key)) {
                                runs.get(i).setText(text.replace(key, ""), 0);
                                XmlCursor cursor = paragraph.getCTP().newCursor();
                                // 在指定游标位置插入表格
                                XWPFTable table = document.insertNewTbl(cursor);
                                //设置表格居中
                                CTJc jc = table.getCTTbl().getTblPr().getJc();
                                if(jc == null){
                                    jc = table.getCTTbl().getTblPr().addNewJc();
                                }
                                jc.setVal(STJc.CENTER);
                                table.getCTTbl().getTblPr().setJc(jc);

                                insertTable2(table, list);
                                break;
                            }
                        }
                    }
                }
            }
        }
        WordUtil.deleteChart(paragraphList);
    }

    /**
     * 将所有包含"${}的段落替换为空
     */
    private static void deleteChart(List<XWPFParagraph> paragraphList){
        if (paragraphList != null && paragraphList.size() > 0) {
            for (XWPFParagraph paragraph : paragraphList) {
                List<XWPFRun> runs = paragraph.getRuns();
                for (int i = 0; i < runs.size(); i++) {
                    String text = runs.get(i).getText(0);
                    if (text != null) {
                        text = text.trim();
                        if(text.contains("$"))
                        runs.get(i).setText("", 0);
                    }
                }
            }
        }
    }



    public static void changeTable(CustomXWPFDocument document, Map<String, Object> textMap,
                                   List<List<Map<Integer,String>>> list) {
        //获取表格对象集合
        List<XWPFTable> tables = document.getTables();
        for (int i = 0; i < tables.size(); i++) {
            //只处理行数大于等于2的表格，且不循环表头
            XWPFTable table = tables.get(i);
            if (table.getRows().size() > 1) {
                //判断表格是需要替换还是需要插入，判断逻辑有$为替换，表格无$为插入
                if (checkText(table.getText())) {
                    List<XWPFTableRow> rows = table.getRows();
                    //遍历表格,并替换模板
                    eachTable(document, rows, textMap);
                } else {
                    insertTable(table, list.get(i));
                }
            }
        }
    }


    /**
     * 遍历表格，包含对图片内容的替换
     *
     * @param rows    表格行对象
     * @param textMap 需要替换的信息集合
     */
    public static void eachTable(CustomXWPFDocument document, List<XWPFTableRow> rows, Map<String, Object> textMap) {
        for (XWPFTableRow row : rows) {
            List<XWPFTableCell> cells = row.getTableCells();
            for (XWPFTableCell cell : cells) {
                //判断单元格是否需要替换
                if (checkText(cell.getText())) {
                    List<XWPFParagraph> paragraphs = cell.getParagraphs();
                    for (XWPFParagraph paragraph : paragraphs) {
                        List<XWPFRun> runs = paragraph.getRuns();
                        for (XWPFRun run : runs) {
//                            run.setText(getValueByKey(run.toString(), textMap),0);

                            Object ob = getValueByKey(run.toString(), textMap);
                            if (ob instanceof String) {
                                run.setText((String) ob, 0);
                            } else if (ob instanceof Map) {
                                run.setText("", 0);

                                Map pic = (Map) ob;
                                int width = Integer.parseInt(pic.get("width").toString());
                                int height = Integer.parseInt(pic.get("height").toString());
                                int picType = getPictureType(pic.get("type").toString());
                                byte[] byteArray = (byte[]) pic.get("content");
                                ByteArrayInputStream byteInputStream = new ByteArrayInputStream(byteArray);
                                try {

                                    String ind = document.addPictureData(byteInputStream, picType);
                                    document.createPicture(run, ind, document.getNextPicNameNumber(picType), width, height);

                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 为表格插入数据，行数不够添加新行
     *
     * @param table     需要插入数据的表格
     * @param tableList 插入数据集合
     */
    public static void insertTable(XWPFTable table, List<String[]> tableList,int start, int end) {
        //遍历表格插入数据
        int insertStart = 0;
        int insertEnd = 0;
        List<XWPFTableRow> rows = table.getRows();
        System.out.println(rows.size());
        for (int i = start; i < rows.size(); i++) {
            XWPFTableRow newRow = table.getRow(i);
            List<XWPFTableCell> cells = newRow.getTableCells();
            for (int j = end; j < cells.size(); j++) {
                XWPFTableCell cell = cells.get(j);
                String value = tableList.get(insertStart)[insertEnd++];
                cell.setText(value);
            }
            insertEnd = 0;
            insertStart++;
        }
    }

    /**
     * 将list中的数据插入表格
     *
     * @param table 需要插入数据的表格
     * @param list  插入数据集合
     */
    public static void insertTable2(XWPFTable table, List<Map<Integer,String>> list) {
        // 动态调整行列大小
        XWPFTableRow row = table.getRow(0);
        int colNum = WordUtil.getMaxMapSize(list);
        for (int col = 1; col < colNum; col++) {
            row.createCell().getCTTc().addNewTcPr();
        }
        List<XWPFTableRow> rows = table.getRows();
        while(rows.size() < list.size())
            table.createRow();

        //遍历表格插入数据
        for (int i = 0; i < rows.size(); i++) {
            Map<Integer,String> map = list.get(i);
            XWPFTableRow newRow = table.getRow(i);
            List<XWPFTableCell> cells = newRow.getTableCells();
            for (int j = 0; j < cells.size(); j++) {
                XWPFTableCell cell = cells.get(j);
                if(map.containsKey(j)){
                    String value = map.get(j);
                    cell.setText(value);
                }
            }
        }
    }

    private static int getMaxMapSize(List<Map<Integer,String>> list){
        int maxSize = 0;
        for (Map<Integer, String> map : list) {
            if(map.size() > maxSize)maxSize = map.size();
        }
        return maxSize;
    }

    /**
     * rows 第一行就是1
     */
    public static void insertTable(XWPFTable table, List<Map<Integer,String>> list) {
        //遍历表格插入数据
        List<XWPFTableRow> rows = table.getRows();
        for (int i = 2; i < rows.size(); i++) {
            // 当前行数据
            Map<Integer,String> map = list.get(i - 2);
            XWPFTableRow newRow = table.getRow(i);
            List<XWPFTableCell> cells = newRow.getTableCells();
            for (int j = cells.size() - 3;  j < cells.size(); j++) {
                XWPFTableCell cell = cells.get(j);
                String value = map.get(j);
                cell.setText(value);
            }
        }
    }

    /**
     * word跨列合并单元格
     */
    public static void mergeCellsHorizontal(XWPFTable table, int row, int fromCell, int toCell) {
        for (int cellIndex = fromCell; cellIndex <= toCell; cellIndex++) {
            XWPFTableCell cell = table.getRow(row).getCell(cellIndex);
            if (cellIndex == fromCell) {
                // The first merged cell is set with RESTART merge value
                cell.getCTTc().addNewTcPr().addNewHMerge().setVal(STMerge.RESTART);
            } else {
                // Cells which join (merge) the first one, are set with CONTINUE
                cell.getCTTc().addNewTcPr().addNewHMerge().setVal(STMerge.CONTINUE);
            }
        }
    }

    /**
     * word跨行并单元格
     */
    public static void mergeCellsVertically(XWPFTable table, int col, int fromRow, int toRow) {
        for (int rowIndex = fromRow; rowIndex <= toRow; rowIndex++) {
            XWPFTableCell cell = table.getRow(rowIndex).getCell(col);
            if (rowIndex == fromRow) {
                // The first merged cell is set with RESTART merge value
                cell.getCTTc().addNewTcPr().addNewVMerge().setVal(STMerge.RESTART);
            } else {
                // Cells which join (merge) the first one, are set with CONTINUE
                cell.getCTTc().addNewTcPr().addNewVMerge().setVal(STMerge.CONTINUE);
            }
        }
    }

    /**
     * 单元格设置字体
     * @param cell
     * @param cellText
     */
    private static void getParagraph(XWPFTableCell cell,String cellText){
        CTP ctp = CTP.Factory.newInstance();
        XWPFParagraph p = new XWPFParagraph(ctp, cell);
        p.setAlignment(ParagraphAlignment.CENTER);
        XWPFRun run = p.createRun();
        run.setText(cellText);
        CTRPr rpr = run.getCTR().isSetRPr() ? run.getCTR().getRPr() : run.getCTR().addNewRPr();
        CTFonts fonts = rpr.isSetRFonts() ? rpr.getRFonts() : rpr.addNewRFonts();
        fonts.setAscii("仿宋");
        fonts.setEastAsia("仿宋");
        fonts.setHAnsi("仿宋");
        cell.setParagraph(p);
    }


    /**
     * 判断文本中时候包含$
     *
     * @param text 文本
     * @return 包含返回true, 不包含返回false
     */
    public static boolean checkText(String text) {
        boolean check = false;
        if (text.indexOf("$") != -1) {
            check = true;
        }
        return check;
    }

    /**
     * 匹配传入信息集合与模板
     *
     * @param value   模板需要替换的区域
     * @param textMap 传入信息集合
     * @return 模板需要替换区域信息集合对应值
     */
    public static Object getValueByKey(String value, Map<String, Object> textMap) {
        Object obj = null;
        Set<Map.Entry<String, Object>> textSets = textMap.entrySet();
        for (Map.Entry<String, Object> textSet : textSets) {
            //匹配模板与替换值 格式${key}
            String key = "${" + textSet.getKey() + "}";
            if (value.indexOf(key) != -1) {
                obj = textSet.getValue();
            }
        }

        if (!(obj instanceof Map)) {
            //模板未匹配到区域替换为空
            if (obj != null && checkText(obj.toString())) {
                obj = "";
            }
        }
        return obj;
    }

    /**
     * 根据图片类型，取得对应的图片类型代码
     *
     * @param picType
     * @return int
     */
    private static int getPictureType(String picType) {
        int res = CustomXWPFDocument.PICTURE_TYPE_PICT;
        if (picType != null) {
            if (picType.equalsIgnoreCase("png")) {
                res = CustomXWPFDocument.PICTURE_TYPE_PNG;
            } else if (picType.equalsIgnoreCase("dib")) {
                res = CustomXWPFDocument.PICTURE_TYPE_DIB;
            } else if (picType.equalsIgnoreCase("emf")) {
                res = CustomXWPFDocument.PICTURE_TYPE_EMF;
            } else if (picType.equalsIgnoreCase("jpg") || picType.equalsIgnoreCase("jpeg")) {
                res = CustomXWPFDocument.PICTURE_TYPE_JPEG;
            } else if (picType.equalsIgnoreCase("wmf")) {
                res = CustomXWPFDocument.PICTURE_TYPE_WMF;
            }
        }
        return res;
    }

    /**
     * 将输入流中的数据写入字节数组
     *
     * @param in
     * @return
     */
    public static byte[] inputStream2ByteArray(InputStream in, boolean isClose) {
        byte[] byteArray = null;
        try {
            int total = in.available();
            byteArray = new byte[total];
            in.read(byteArray);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (isClose) {
                try {
                    in.close();
                } catch (Exception e2) {
                    System.out.println("关闭流失败");
                }
            }
        }
        return byteArray;
    }

}
